#ifndef BEHAVIOR_H
#define BEHAVIOR_H

#include "pixeltype.h"
#include "coord.h"
#include <string>

#define DUDE_MEMORY_SIZE 128
#define LIBRARY_SIZE 128
#define COM_SIZE 32
#define DEBUG_OUTPUT_MAX_SIZE 2048

class Map;

struct Com
{
    enum Flags {
        BOOK1 = 0,
        BOOK2 = 1,
        BOOK3 = 2,
        BOOK4 = 3,
        
        READ  = 0,
        WRITE = 4
    };

	int flag = 0;
	char data[COM_SIZE] = {0};
};

struct Action
{
    enum Type {
        MOVE,
        ATTACK,
        PICK,
        PUT,
        WORK,
        WAIT,
        COMMUNICATE
    };
    
    Com com_data;
	Type type;
	Dir dir;
};

struct Info
{
    virtual bool getSuccess() const = 0;
    virtual PixelType getInventory() const = 0;
    virtual const Com* getCom() const = 0;
    virtual PixelType getNear(Dir d) const = 0;
    virtual int getInfo(Dir d) const = 0;
};

/**
 * must be named "think"
 *
 * this function should initialize the "action" struct,
 * you can read and write in the dude's "memory"
 * and you have access to additionnal information through "info"
 */
typedef void (*BehaviorFunction)(Action *action, char *memory, const Info *info);

/**
 * must be named "debugOutput"
 *
 * This function is facultative, if you implement it,
 * you must write a valid string in "outputString",
 * you have a read access on the targeted dude's memory.
 *
 * The outputString has a max size of DEBUG_OUTPUT_MAX_SIZE.
 */
typedef void (*DebugBehaviorFunction)(char *outputString, const char *memory);

#endif // BEHAVIOR_H
